; ---------------------------------------------------------------------------------------------------
; IMPLEMENTACAO DE UM RELOGIO DE TEMPO REAL USANDO INTERRUPCOES DE TIMER       Spinelli - 02/09/2011
;
; STATUS:FUNCIONA
;
; ---------------------------------------------------------------------------------------------------
; ALTERACOES
;
; 01/10/2014 - Inclusao de todas as subrotinas e macros na pasta do projeto.
;              Os arquivos externos "delay_us.inc", "delay_ms.inc", "ram.inc" e "macros.inc" 
;              que contem todos os modulos necessarios para o projeto, foram movidos para
;              dentro da pasta do projeto.
;
; 29/08/2011 - Alterado os PORTx para LATx para corrigir problema de leitura dos botoes.
;              Tempo de varredura do display alterado para inCT=0x9F00.
;
; 01/09/2011 - Tempo de debounce das teclas reduzido para 40mS para a digitacao ficar mais rapida.
;
; 02/09/2011 - Passou a operar com clock externo de 24MHZ.
;              
;
; ---------------------------------------------------------------------------------------------------
;
; CARACTERISTICAS E FUNCOES DO RELOGIO:
; 1 - Relogio de 24 horas com exibicao no display de 7 segmentos FTTL-655S
; 2 - Funcao para acerto do relogio 
; 3 - Funcao para acerto do alarme 
; 4 - Funcao de interrupcao do alarme
; 5 - Funcao para definicao de intevalo de tempo de repeticao do alarme.
; 6 - Funcao para definicao de tempo de acionamento de um rele para controle de carga externa.
; 
;
; O RELOGIO DE TEMPO REAL
; -----------------------
; O relogio de tempo real foi implementado com um oscilador externo de 32KHZ nas portas T1OSI e T1OSO, 
; conforme descrito no data sheet do 18F4520, que sera usado pelo TIMER1 na temporizacao do circuito. 
; Os capacitores do clock sao criticos e, para montagem no protoboard, foi necessario usar valores de 
; 2,7pF a 10pF para conseguir uma temporizacao adequada (o circuito foi montado com 10pF).
; Os registradores TMR1H:TMR1L do TIMER1 serao inicializados com o valor 32768 que, a uma frequencia de
; 32KHZ, fornece um estouro de contagem preciso a cada 1 segundo. A cada estouro, uma interrupcao de TIMER1
; sera disparada e a rotina ISR (Interruption Service Routine) do TIMER1 atualizara as horas.
;
;
; VARREDURA DO DISPLAY DE 7 SEGMENTOS
; -----------------------------------
; O TIMER0 sera usado para multiplexar o display. O valor de contagem sera definido no registrador
; "intCT". Esse valor deve ser ajustado para o display nao piscar. A cada iterrupcao de TIMER0, a rotina
; ISR de TIMER0 sera disparada para atualizar o display.
;
;
; BOTOES DE FUNCOES
; -----------------
; Botoes usados no PORTC<7,6,5,4,2> e PORTE<2,1,0>:
; Os botoes devem ser conectados, de um lado, diretamente aos PORTs e, de outro lado, ao ground. 
; Os PORTC<7,6,5,4,2> e PORTE<2,1,0> devem ser conectados ao VCC com resistores de 1K, portanto, ao
; pressionar os botoes, os PORTs serao levados ao nivel baixo, significando que produzirao uma logica 
; negativa. As seguintes portas serao usadas para os botoes de funcao:
;
; C2 --> para entrar em modo de acerto do alarme; esse botao deve permanecer pressionado enquanto
;        se acerta as horas e os minutos do alarme.
; C4 --> avanca as horas   (manter C6 pressionado)
; C5 --> avanca os minutos (manter C6 pressionado)
; C6 --> para entrar em modo de acerto de horas; esse botao deve permanecer pressionado enquanto
;        se acerta as horas e os minutos.
; C7 --> definicao da repeticao do alarme; manter esse botao pressionado enquanto definir o
;        intervalo (HH:MM) de repeticao do alarme; se definido HH:MM = 00:00 nao havera repeticao.
; E0 --> interrupcao do alarme
; E1 --> definir intervalo durante o qual o rele permanece ligado
; E2 --> definir intervalo durante o qual o rele permanece desligado
;
;
;
; ACIONAMENTO DE UM DISPOSITIVO EXTERNO
; -------------------------------------
; Um dispositivo externo, como um ventilador, um radio ou uma lampada, pode ser acionado atraves de
; um rele conectado ao PORTA<0>. 
; O dispositivo podera ser programado para permanecer ligado durante um intervalo de tempo definido com a
; funcao "Intervalo ligado" do botao em PORTE<1>. 
; O dispositivo podera ser programado para permanecer desligado durante um intervalo de tempo  definido
; com a funcao "Intervalo desligado" do botao em PORTE<2>.
; Para desativar a programacao de acionamento do dispositivo externo, basta zerar o "Intervalo ligado"
; com a funcao em PORTE<1>.
;
;
; ACERTO DAS HORAS E MINUTOS DO RELOGIO
; -------------------------------------
; Para acertar as horas, segure pressionado o botao do PORTC<6> e ao mesmo tempo pressione o botao
; do PORTC<4>. Os valores que estao sendo modificados serao mostrados nos dois digitos a esquerda
; do display.
; Para acertar os minutos, segure pressionado o botao do PORTC<6> e ao mesmo tempo pressione o botao
; do PORTC<5>. Os valores que estao sendo modificados serao mostrados nos dois digitos a direita
; do display.
;
;
; O ALARME
; --------
; O relogio pode ser programado para soar um alarme na hora desejada. 
; Ao chegar a hora programada, o alarme comecara a soar e assim permanacera durante 1 minuto.
; Para interromper o alarme, pressione o botao em PORTE<0>.
; O alarme tambem pode ser programado para ser repetido periodicamente.
;
;
; PROGRAMACAO DO ALARME
; ---------------------
; Para acertar as horas do alarme, segure pressionado o botao do PORTC<2> e ao mesmo tempo pressione o botao
; do PORTC<4>. Os valores que estao sendo modificados serao mostrados nos dois digitos a esquerda
; do display.
; Para acertar os minutos do alarme, segure pressionado o botao do PORTC<2> e ao mesmo tempo pressione o botao
; do PORTC<5>. Os valores que estao sendo modificados serao mostrados nos dois digitos a direita
; do display.
;
;
; PROGRAMACAO DA REPETICAO PERIODICA DO ALARME
; --------------------------------------------
; O alarme pode ser programado para ser repetido periodicamente.
; Para programar as horas do intervalo de tempo de repeticao do alarme, pressione o botao em PORTC<7> e ao
; mesmo tempo pressione o botao em PORTC<4>. Os valores que estao sendo modificados serao mostrados nos dois 
; digitos a esquerda do display.
; Para programar os minutos do intervalo de tempo de repeticao do alarme, pressione o botao em PORTC<7> e ao
; mesmo tempo pressione o botao em PORTC<5>. Os valores que estao sendo modificados serao mostrados nos dois 
; digitos a direita do display.
;
;
; INTERRUPCAO DO ALARME
; ---------------------
; Se o alarme estiver soando e desejar interrompe-lo, pressione o botao em PORTE<0>. O alarme deixara
; de soar e so vai faze-lo na proxima hora programada.
;
;
;
; NOTAS:
; 1 - Ate esta versao nao foi possivel gravar a EEPROM com o valor do alarme durante a programacao do chip,
;     fazendo com que o valor do alarme deva ser posicionado depois do circuito ser ligado.
;
;
;
; DESCRICAO DO HARDWARE DO PIC
; ----------------------------
;
; MCLR................... - reset e ICSP
; A0..................... - Controle liga/desliga do rele
; A3..................... - Catodo 1
; A4..................... - Catodo 2
; A5..................... - BUZZER do alarme
; B7 a B0................ - aos anodos do display (ver tabela abaixo) e ICSP (B7, B6 e B5)
; C2..................... - Botao de acerto do alarme
; C3..................... - Dois pontos do display
; C4..................... - Botao de avanco das horas
; C5..................... - Botao de avanco dos minutos
; C6..................... - Botao de acerto das horas
; C7..................... - Botao de definicao da repeticao do alarme
; D7 a D2................ - aos anodos do display (ver tabela abaixo)
; E0..................... - Interruptor do alarme quando este estiver soando
; E1..................... - Botao de definicao do intervalo de tempo de rele ligado
; E2..................... - Botao de definicao do intervalo de tempo de rele desligado
;
;
; CONTROLE DOS CATODOS DO DISPLAY
; -------------------------------
; Os I/Os A3 e A4 deverao ser ligados as bases dos transistores BC337 com resistores de 4K7. 
; Os coletores dos transistores devem ser ligados aos pinos 1 e 2 do display e seus emissores diretamente ao ground.
;
;
; CONTROLE DOS ANODOS DO DISPLAY
; ------------------------------
; O I/O C3 do PIC deve ser ligado ao pino 30 do display atraves de um resistor de 100R para acender os dois pontos  
; centrais do display.
;
;
; CONTROLE DOS SEGMENTOS DO DISPLAY
; ---------------------------------
; A tabela abaixo mostra as conexoes para acender os segmentos do display. 
; Essa tabela define a estrutura do bloco de controle descritor de segmentos "segmap"
; Todas as conexoes, entre o PIC e o display, devem ser feitas com resistores de 100R.
;
; -------+------+---------+
;        |      | CATODO  | 
;        |      +---------+------------------------------------------------------------------------------------------- 
; DIGITO | SEGM |  1    2 |   6    7    8    9   10   12    13    15    16    17    18    19    20    21  ---> DISPLAY
;        |      | A3   A4 |  B7   B6   B5   B4   B3   B2    B1    B0    D7    D6    D5    D4    D3    D2  ---> PIC     
; -------+------+---------+-------------------------------------------------------------------------------------------      
;        |  a   |  1      |        1 
;        |  b   |       1 |   1
;        |  c   |       1 |                  1
;   DIG1 |  d   |  1      |             1
;        |  e   |       1 |             1
;        |  f   | ....... |   ....................... NAO DEFINIDO NO DISPLAY ..........................
;        |  g   |       1 |        1
; -------+------+---------+-------------------------------------------------------------------------------
;        |  a   |       1 |                                  1
;        |  b   |       1 |                       1
;        |  c   |       1 |                            1
;   DIG2 |  d   |  1      |                            1
;        |  e   |  1      |                  1
;        |  f   |  1      |                                  1
;        |  g   |  1      |                       1
; -------+------+---------+-------------------------------------------------------------------------------
;        |  a   |  1      |                                        1
;        |  b   |  1      |                                              1
;        |  c   |  1      |                                                    1
;   DIG3 |  d   |       1 |                                                    1
;        |  e   |       1 |                                                          1
;        |  f   |       1 |                                        1
;        |  g   |       1 |                                              1
; -------+------+---------+-------------------------------------------------------------------------------
;        |  a   |       1 |                                                                            1          
;        |  b   |       1 |                                                                1 
;        |  c   |       1 |                                                                      1 
;   DIG4 |  d   |  1      |                                                                      1
;        |  e   |  1      |                                                         1
;        |  f   |  1      |                                                                            1
;        |  g   |  1      |                                                                1
; -------+------+---------+-------------------------------------------------------------------------------
;
;
	LIST P=18F4520
	#include <P18F4520.INC>
;
; ---------------------------------------------------------------------------------------------
; CONFIGURATION BITS ou FUSES
; ---------------------------------------------------------------------------------------------
    CONFIG WDT   = OFF
	CONFIG MCLRE = ON
	CONFIG DEBUG = OFF
	CONFIG LVP   = OFF 
	CONFIG OSC   = HS                            ; oscilador externo
    CONFIG CP0   = OFF
    CONFIG CP1   = OFF
    CONFIG CP2   = OFF
    CONFIG CP3   = OFF
    CONFIG CPB   = OFF
    CONFIG CPD   = OFF
    CONFIG WRT0  = OFF
    CONFIG WRT1  = OFF
    CONFIG WRT2  = OFF
    CONFIG WRT3  = OFF
    CONFIG WRTC  = OFF
    CONFIG WRTB  = OFF
    CONFIG WRTD  = OFF
; ---------------------------------------------------------------------------------------------
             #include     "macros.inc"           ; arquivo de origem --> "macros.inc"
; ---------------------------------------------------------------------------------------------
             UDATA       0x080
;
flag1        RES         1                       ; flag1 do programa
fhoras       equ         0                       ; flag1<0> --> display/set horas
falarme      equ         1                       ; flag1<1> --> display/set alarme
alarmon      equ         2                       ; flag1<2> --> alarm liga/desliga
frepeat      equ         3                       ; flag1<3> --> display/set repeticao 
fproxrpt     equ         4                       ; flag1<4> --> rotina prox_repete em andamento 
frelesta     equ         5                       ; flag1<5> --> status do rele 0-desligado  1-ligado 
falarmstop   equ         6                       ; flag1<6> --> 0-continua soando  1-interrompe o alarme
;
flag2        RES         1                       ; flag2 do programa
frelec       equ         0                       ; flag2<0> --> definindo ciclos do rele
freleii      equ         1                       ; flag2<1> --> definindo intervalo de rele ligado
freleid      equ         2                       ; flag2<2> --> definindo intervalo de rele desligado
;
dechour      RES         1                       ; buffer das horas
dechour1     RES         1
decmin       RES         1
decmin1      RES         1
decsec       RES         1 
decsec1      RES         1
;
al_hour      RES         1                       ; buffer do alarme
al_hour1     RES         1
al_min       RES         1
al_min1      RES         1
;
rpt_hour     RES         1                       ; buffer para repeticao do alarme
rpt_hour1    RES         1
rpt_min      RES         1
rpt_min1     RES         1
;
hh           RES         1
mm           RES         1
ss           RES         1
;
Wsave        RES         1                       ; save area
Ssave        RES         1
Wsave1       RES         1
Ssave1       RES         1
Sfsr1_1      RES         1
Sfsr0_1      RES         1
;
delay_temp   RES         1                       ; para a subrotina de delay
delay_wrk1   RES         1
delay_wrk2   RES         1
;
ledsec       RES         1
;
dig_atual    RES         1                       ; para a rotina de varredura do display                   
bitmap       RES         1
segnum       RES         1
segptr       RES         1
segdesc      RES         1
catodo       RES         1
port1        RES         1
port2        RES         1
;
alhour       RES         1                       ; registradores para calculos de tempo
almin        RES         1
rpthour      RES         1
rptmin       RES         1
;
R0           RES         1                       ; registradores de trabalho
R1           RES         1
R2           RES         1
R14          RES         1
R15          RES         1
;
relelhd      RES         1                       ; dezenas de horas intervalo de tempo de rele ligado
relelhu      RES         1                       ; unidades de horas intervalo de tempo de rele ligado
relelmd      RES         1                       ; dezenas de minutos intervalo de tempo de rele ligado  
relelmu      RES         1                       ; unidades de minutoa intervalo de tempo de rele ligado
;
reledhd      RES         1                       ; dezenas de horas intervalo de tempo de rele desligado
reledhu      RES         1                       ; unidades de horas intervalo de tempo de rele desligado
reledmd      RES         1                       ; dezenas de minutos intervalo de tempo de rele desligado
reledmu      RES         1                       ; unidades de minutos intervalo de tempo de rele desligado
;
relelch      RES         1                       ; horas contador de tempo de rele ligado
relelcm      RES         1                       ; minutos contador de tempo de rele ligado
relelcs      RES         1                       ; segundos contador de tempo de rele ligado
;
reledch      RES         1                       ; horas contador de tempo de rele desligado
reledcm      RES         1                       ; minutos contador de tempo de rele desligado
reledcs      RES         1                       ; segundos contador de tempo de rele desligado
;
cmdbuff1     RES         1                       ; buffer de comandos 1 para configuracao do relogio
cmdbuff2     RES         1                       ; buffer de comandos 2 para configuracao do relogio
PABuff       RES         1                       ; buffer do PORTA
;
SVStatus     RES         1
svct         RES         1                       ; save area para registradores aritmeticos
SV_MATH	     RES       .26                       ; 26 registradores aritmeticos
;
             FAMILIAS                            ; arquivo de origem --> "macros.inc"
;
             #include "ram.inc"                  ; arquivo de origem --> "ram.inc"
;
; ---------------------------------------------------------------------------------------------
PORTA_CMD1   equ         PORTC                   ; porta de comandos 1
PORTA_CMD2   equ         PORTE                   ; porta de comandos 2
CLOCK        equ         24000000                ; frequencia de clock
carga        equ         .32768                  ; interrupcoes de TIMER1 a cada 1 segundo.
ka2          equ         4                       ; catodo 2
KEY_DEBOUNCE equ         .50                     ; retardo para botoes em milisegundos
intCT        equ         0x8B00                  ; 0xAA00 ---> TIMER0 varredura do display
;
;
; ---------------------------------------------------------------------------------------------
;
pwron  CODE  0x0000
       goto  inicio
intr   CODE 0x0004
       retfie
intrf  CODE 0x008
       goto  RTCisr
; ---------------------------------------------------------------------------------------------
MAIN   CODE
;
; ---------------------------------------------------------------------------------------------
inicio   movlw  0x0F                             ;  
         movwf  ADCON1                           ; todas as portas sao I/O digital
;
         clrf   LATA
         clrf   TRISA

         clrf   LATB
         clrf   TRISB                            ; saida de dados para os pinos do display

         clrf    LATC
         movlw   b'11110100'                     ; PORTC<7,6,5,4,2> sao entradas 
         movwf   TRISC                           ; ver comentarios de descricao do hardware

         clrf    LATD
         movlw   b'00000000'
         movwf   TRISD

         clrf    LATE                           ; PORTE<2,1,0> - intervalo e ciclos do rele
         movlw   b'00000111'
         movwf   TRISE

; ---------------------------------------------------------------------------------------------
; POSICIONA AS HORAS E O ALARME QUANDO O CHIP FOR LIGADO OU SOFRER UM RESET.
; Isso esta sendo feito porque nao ha bateria de nobreak, assim, o circuito vai apagar
; toda vez que for religado. Use os botoes para acertar as horas e o alarme apos o circuito
; ser reiniciado.
; ---------------------------------------------------------------------------------------------
           movlw    .1                           ; ao iniciar o programa, posiciona alarme
           movwf    al_hour                      ; em 16:45 para saber a hora de sair do laboratorio
           movlw    .6
           movwf    al_hour1
           movlw    .4
           movwf    al_min
           movlw    .5
           movwf    al_min1
;
           movlw    .0                           ; ao iniciar o programa, posiciona as horas
           movwf    dechour                      ; em 09:00:00 
           movlw    .8                            
           movwf    dechour1                      
           movlw    .5
           movwf    decmin 
           movlw    .9
           movwf    decmin1
           movlw    .5
           movwf    decsec 
           movlw    .5 
           movwf    decsec1
;
           movlw    .0                           ; ao iniciar o programa,  
           movwf    rpt_hour                     ; nao estabelecer repeticao do alarme
           movlw    .0                           ; para nao ficaer enchendo o saco
           movwf    rpt_hour1
           movlw    .0
           movwf    rpt_min
           movlw    .0
           movwf    rpt_min1

; ---------------------------------------------------------------------------------------------
           movlw    .44                          ; limpa area de trabalho          
           movwf    R1
           lfsr     FSR0,relelhd
clr1       clrf     POSTINC0
           decfsz   R1,f
           goto     clr1
;
           clrf     relelhd                      ; tempo ligado
           clrf     relelhu
           clrf     relelmd
           clrf     relelmu
;
           clrf     reledhd                      ; tempo desligado
           clrf     reledhu
           clrf     reledmd
           clrf     reledmu
;
           movlw    .59                          ; segundos do cronometro de rele ligado
           movwf    relelcs                      ; inicializar com 59, senao...
;
           call     flagsoff                     ; limpa todos os flags de funcoes ativas       
;
           clrf     PABuff                       ; limpa o buffer do PORTA
;
           clrf     ledsec                       ; apaga dois pontos centrais
           bsf      ledsec,3                     ; 
;
           clrf     catodo                       ; catodo = 0000 0000
           bsf      catodo,ka2                   ; catodo = 0001 0000
;
           call     flagsoff                     ; desliga todos os flags de funcoes ativas
           bcf      flag1,alarmon                ; desliga flag do alarme
           bsf      flag1,fhoras                 ; liga o flag de horas por default
           lfsr     FSR1,decmin1                 ; por default, aponta varredura para as horas
;
           bsf      flag1,frelesta               ; deixa ligado o ciclo do rele
;
; ---------------------------------------------------------------------------------------------
;    CONFIGURA O TIMER0 - Varredura do display
; ---------------------------------------------------------------------------------------------
tmr0init   bcf        T0CON,TMR0ON                 ; TIMER0 -> desligado 
           bcf        T0CON,T0CS                   ; TIMER0 -> timer mode
           bcf        T0CON,T08BIT                 ; TIMER0 -> 16 bits
           movlw      HIGH(intCT)
           movwf      TMR0H
           movlw      LOW(intCT)
           movwf      TMR0L
           bsf        INTCON,TMR0IE                ; habilita interrupcao de TIMER0
           bsf        T0CON,TMR0ON                 ; TIMER0 -> ligado
; ---------------------------------------------------------------------------------------------
;    CONFIGURA O TIMER1 - RTC (relogio de tempo real)
; ---------------------------------------------------------------------------------------------
RTCinit    bcf        T1CON,TMR1ON                 ; desliga o TIMER1
           clrf       TMR1L                        ; limpa TMR1L para nao ter perigo de estourar
           movlw      HIGH(carga)                  ; pre carga
           movwf      TMR1H                        ; para 1 segundo
           movlw      LOW(carga)                   ; pre carga
           movwf      TMR1L                        ; para 1 segundo
           bsf        T1CON,T1OSCEN                ; habilita oscilador
           bsf        T1CON,T1SYNC                 ; nao sincroniza o clock externo
           bsf        T1CON,TMR1CS                 ; seleciona clock externo
           bsf        PIE1, TMR1IE                 ; habilita interrupcao de TIMER1
           bsf        INTCON,PEIE                  ; habilita interrupcoes de perifericos
           bsf        INTCON,GIE                   ; habilita interrupcoes gerais
           bsf        T1CON,TMR1ON                 ; liga o TIMER1

; ---------------------------------------------------------------------------------------------
;             ROTINA PRINCIPAL
; ---------------------------------------------------------------------------------------------
ver_comando   call     _debounce                 ; espera estabilizar 
              movff    PORTA_CMD1,cmdbuff1       ; le a porta de comandos 1
              call     _debounce                 ; espera estabilizar 
;
              movff    PORTA_CMD2,cmdbuff2       ; le a porta de comandos 2
              call     _debounce                 ; espera estabilizar 
;
              btfss    cmdbuff1,6                ; testa botao de acerto de horas
              goto     set_acerto                ; se pressionado, vai acertar as horas

              btfss    cmdbuff1,2                ; testa botao de acerto de alarme
              goto     set_alarme                ; se pressionado, vai acertar o alarme

              btfss    cmdbuff1,7                ; testa botao de definicao de repeticao do alarme
              goto     set_repeat                ; se pressionado, definir repeticao
;
              btfss    cmdbuff2,0                ; testa botao interruptor do alarme
              goto     set_alarmoff              ; se pressionado, vai imterromper alarme
;
              btfss    cmdbuff2,1                ; testa botao de definicao de intervalo de rele ligado
              goto     set_releintl              ; se pressionado, vai definir intervalo de rele ligado
;
              btfss    cmdbuff2,2                ; testa botao de definicao de intervalo de rele desligado
              goto     set_releintd              ; se pressionado, vai definir intervalo de rele desligado
; ---------------------------------------------------------------------------------------------
;             SE NAO HA BOTAO PRESSIONADO, SEGUE MOSTRANDO A HORA NORMAL
;             E VERIFICANDO SE ESTA NA HORA DO ALARME SOAR
; ---------------------------------------------------------------------------------------------
              call     flagsoff                  ; desliga flags 
              bsf      flag1,fhoras              ; liga o flag de horas por default
              call     proc_alarme               ; vai verificar se esta na hora do alarme soar
              btfsc    flag1,alarmon             ; se o gatilho do alarme esta armado,
              call     soar_alarme               ; entao vai soar o alarme, senao...
              call     proc_rele                 ; executa procedimento do rele
              goto     ver_comando               ; senao, volta a ler a porta de comandos
; ---------------------------------------------------------------------------------------------
; INTERRUPTOR DO ALARME
; Esta funcao desliga o som do alarme se este estiver ativo.
; Esta funcao nao para definitivamente o alarme. O alarme voltara a soar na proxima vez.
; ---------------------------------------------------------------------------------------------
set_alarmoff  bsf      flag1,falarmstop          ; pede para interromper o alarme
              goto     ver_comando               ; ler comando
; ---------------------------------------------------------------------------------------------
; DEFINICAO DO INTERVALO DE TEMPO DE RELE LIGADO
; Se o intervalo for zero, entao o rele estara permanentemente desativado.
; Esta rotina e acionada pelo botao conectado ao PORTE<1>.
; ---------------------------------------------------------------------------------------------
set_releintl  call     flagsoff                  ; desliga todos os flags de funcoes ativas
              bsf      flag2,freleii             ; liga modificacao de intervalo ligado
              lfsr     FSR1,relelhd              ; endereco das unidades de horas
              btfss    cmdbuff1,4                ; testa botao de avanco de horas
              call     adxyzhora                 ; se pressionado, vai incrementar as horas
              lfsr     FSR1,relelmd              ; endereco das unidades dos minutos
              btfss    cmdbuff1,5                ; testa botao de avanco de minutos
              call     adxyzmin                  ; se pressionado, vai incrementar minutos
              goto     ver_comando               ; ler comando
; ---------------------------------------------------------------------------------------------
; DEFINICAO DE INTERVALO DE TEMPO DE RELE DESLIGADO
; Esta rotina e acionada pelo botao conectado ao PORTE<2>.
; ---------------------------------------------------------------------------------------------
set_releintd  call     flagsoff                  ; desliga todos os flags de funcoes ativas
              bsf      flag2,freleid             ; liga modificacao de intervalo desligado  
              lfsr     FSR1,reledhd              ; endereco das unidades de horas
              btfss    cmdbuff1,4                ; testa botao de avanco de horas
              call     adxyzhora                 ; se pressionado, vai incrementar as horas
              lfsr     FSR1,reledmd              ; endereco das unidades dos minutos
              btfss    cmdbuff1,5                ; testa botao de avanco de minutos
              call     adxyzmin                  ; se pressionado, vai incrementar minutos
              goto     ver_comando               ; ler comando        
; ---------------------------------------------------------------------------------------------
; ACERTO DO INTERVALO PARA REPETICAO DE ALARME
; O alarme pode se repetir indefinidamente usando o intervalo de tempo (HH:MM)
; definido neste segmento. O limite do intervalo de tempo deve ser 23 horas e 59 
; minutos. Entrar 00:00 para nao definir repeticao do alarme.
; Esta rotina e acionada pelo botao conectado ao PORTC<7>.
; ---------------------------------------------------------------------------------------------
set_repeat    call     flagsoff                  ; desliga todos os flags de funcoes ativas
              bsf      flag1,frepeat             ; liga display/set repeticao,
              lfsr     FSR1,rpt_hour             ; aponta dezenas das horas
              btfss    cmdbuff1,4                ; testa botao de avanco de horas
              call     adxyzhora                 ; vai incrementar as horas
              lfsr     FSR1,rpt_min              ; aponta dezenas dos minutos
              btfss    cmdbuff1,5                ; testa botao de acerto dos minutos
              call     adxyzmin                  ; vai incrementar minutos
              goto     ver_comando               ; senao, sai fora
; ---------------------------------------------------------------------------------------------
; ACERTO DE HORAS
; Esta rotina e acionada pelo botao conectado ao PORTC<6>.
; Esta rotina desabilita temporariamente a interrupcaode TIMER1 para poder acertar as horas.
; ---------------------------------------------------------------------------------------------
set_acerto    bcf      T1CON,TMR1ON              ; desliga o TIMER1
              bcf      PIE1, TMR1IE              ; desabilita interrupcao de TIMER1

              call     flagsoff                  ; desliga todos os flags de funcoes ativas
              bsf      flag1,fhoras              ; liga o flag de modificacao/exibicao das horas
              lfsr     FSR1,dechour              ; aponta dezenas das horas
              btfss    cmdbuff1,4                ; testa botao de avanco de horas
              call     adxyzhora                 ; vai incrementar as horas
              lfsr     FSR1,decmin               ; aponta dezenas dos minutos
              btfss    cmdbuff1,5                ; testa botao de acerto dos minutos
              call     adxyzmin                  ; vai incrementar minutos

              movwf    HIGH(carga)               ; pre carga do TIMER1
              movwf    TMR1H                     ; para 1 segundo
              movwf    LOW(carga)                ; pre carga
              movwf    TMR1L                     ; para 1 segundo
              bcf      PIR1, TMR1IF              ; limpa flag de interrupcao do TIMER1
              bsf      PIE1, TMR1IE              ; habilita interrupcao de TIMER1
              bsf      T1CON,TMR1ON              ; liga o TIMER1

              goto     ver_comando               ; volta a ler a porta de comandos
; ---------------------------------------------------------------------------------------------
; ACERTO DO ALARME
; Esta rotina e acionada pelo botao conectado ao PORTC<2>.
; ---------------------------------------------------------------------------------------------
set_alarme    call     flagsoff                  ; desliga todos os flags de funcoes ativas
              bsf      flag1,falarme             ; liga o flag de modificacao/exibicao do alarme
              lfsr     FSR1,al_hour
              btfss    cmdbuff1,4                ; testa botao de avanco de horas para alarme
              call     adxyzhora
              lfsr     FSR1,al_min
              btfss    cmdbuff1,5                ; testa botao de acerto dos minutos
              call     adxyzmin
              ;call     salvalarme               ; isso nao tem funcao
              goto     ver_comando               ; volta a ler a porta de comandos
; ---------------------------------------------------------------------------------------------
; OPERACAO DO RELE
; ---------------------------------------------------------------------------------------------
proc_rele     movf     relelhd,w                 ; pega as dezenas das horas
              addwf    relelhu,w                 ; soma com as unidades das horas
              addwf    relelmd,w                 ; soma com as dezenas dos minutos
              addwf    relelmu,w                 ; soma com as unidades dos minutos
              btfsc    STATUS,Z                  ; verifica se o resultado deu zero
              return                             ; se deu zero, entao o rele esta desativado

              btfsc    flag1,frelesta            ; verifica se esta no ciclo ligado
              goto     rele_ciclo_l              ; rele deve ser ligado
              goto     rele_ciclo_d              ; rele deve ser desligado

rele_ciclo_l  call     rele_on                   ; mantem o rele ligado
              movff    relelhd,DIGIT1            ; dezena das horas
              movff    relelhu,DIGIT2            ; unidade das horas
              movlw    .2                        ; tamanho
              call     dec2bin                   ; converte a binario
              movf     relelch,w                 ; horas do cronometro de ciclo ligado
              subwf    REGA0,w                   ; subtrai hora cronometro da hora do ciclo ligado
              btfss    STATUS,Z                  ; verifica o resultado
              return                             ; se diferente de zero, sai fora, senao...
              movff    relelmd,DIGIT1            ; dezena dos minutos
              movff    relelmu,DIGIT2            ; unidade dos minutos
              movlw    .2                        ; tamanho
              call     dec2bin                   ; converte a binario
              movf     relelcm,w                 ; minutos do cronometro de ciclo ligado
              subwf    REGA0,w                   ; subtrai minutos do cronometro dos minutos do ciclo ligado
              btfss    STATUS,Z                  ; verifica o resultado
              return                             ; se diferente de zero, sai fora, senao...
              call     zeracrono                 ; zera os cronometros
              bcf      flag1,frelesta            ; desabilita ciclo ligado  
              call     rele_off                  ; desliga o rele
              return                             ; fim
;
rele_ciclo_d  call     rele_off                  ; mantem o rele desligado
              movff    reledhd,DIGIT1            ; dezena daas horas
              movff    reledhu,DIGIT2            ; unidade das horas
              movlw    .2                        ; tamanho
              call     dec2bin                   ; converte a binario
              movf     reledch,w                 ; horas do cronometro de ciclo desligado
              subwf    REGA0,w                   ; subtrai hora cronometro da hora do ciclo desligado
              btfss    STATUS,Z                  ; verifica o resultado
              return                             ; se diferente de zero, sai fora, senao...
              movff    reledmd,DIGIT1            ; dezena dos minutos
              movff    reledmu,DIGIT2            ; unidade dos minutos
              movlw    .2                        ; tamanho
              call     dec2bin                   ; converte a binario
              movf     reledcm,w                 ; minutos do cronometro de ciclo desligado
              subwf    REGA0,w                   ; subtrai minutos cronometro dos minutos do ciclo desligado
              btfss    STATUS,Z                  ; verifica o resultado
              return                             ; se diferente de zero, sai fora, senao...
              call     zeracrono                 ; zera os cronometros
              bsf      flag1,frelesta            ; habilita o ciclo ligado
              call     rele_on                   ; liga o rele
              return                             ; fim
;
rele_on       bsf     PABuff,0                   ; liga o rele
              movff   PABuff,LATA               ; 
              return                             ;

rele_off      bcf     PABuff,0                   ; desliga o rele
              movff   PABuff,LATA               ; 
              return                             ; 

zeracrono     clrf     relelch                   ; zera o cronometro
              clrf     relelcm                   ; zera o cronometro
              clrf     relelcs                   ; zera o cronometro
              clrf     reledch                   ; zera o cronometro
              clrf     reledcm                   ; zera o cronometro
              clrf     reledcs                   ; zera o cronometro
              return
; ---------------------------------------------------------------------------------------------
;             INCREMENTA HORA
; Entrada: FSR1 aponta para as dezenas das horas
; Saida:   Horas apontadas por FSR1 incrementadas
; ---------------------------------------------------------------------------------------------
adxyzhora     movff    POSTINC1,DIGIT1           ; pega dezenas de horas
              movff    POSTINC1,DIGIT2           ; pega unidades do horas
              movlw    .2                        ; tamanho
              call     dec2bin                   ; converte a binario
              incf     REGA0,f                   ; incrementa a hora
              movlw    .23                       ; 23
              cpfsgt   REGA0                     ; verifica o limite
              goto     adxyzrst                  ; se dentro do limite, vai devolver resultado
              movf     POSTDEC1,w                ; so para decrementar o endereco
              clrf     POSTDEC1                  ; zera as unidades dos horas
              clrf     POSTDEC1                  ; zera as dezenas dos horas
              return                             ; fim
; ---------------------------------------------------------------------------------------------
;             INCREMENTA MINUTOS
; Entrada: FSR1 aponta para as dezenas dos minutos
; Saida:   Minutos apontados por FSR1 incrementados
; ---------------------------------------------------------------------------------------------
adxyzmin      movff    POSTINC1,DIGIT1           ; pega dezenas de minutos
              movff    POSTINC1,DIGIT2           ; pega unidades do minutos
              movlw    .2                        ; tamanho
              call     dec2bin                   ; converte a binario
              incf     REGA0,f                   ; incrementa a minutos
              movlw    .59                       ; 59
              cpfsgt   REGA0                     ; verifica o limite
              goto     adxyzrst                  ; se dentro do limite, vai devolver resultado
              movf     POSTDEC1,w                ; so para decrementar o endereco
              clrf     POSTDEC1                  ; zera as unidades dos minutos
              clrf     POSTDEC1                  ; zera as dezenas dos minutos
              return                             ; fim
; ---------------------------------------------------------------------------------------------
; SAIDA DA ROTINA DE INCREMENTO DE HORAS E MINUTOS
; ---------------------------------------------------------------------------------------------
adxyzrst      call     bin2dec                   ; converte valor calculado a decimal
              movf     POSTDEC1,w                ; so para decrementar o endereco
              movff    DIGIT10,POSTDEC1          ; devolve unidade dos minutos incrementados
              movff    DIGIT9,POSTDEC1           ; devolve dezenas dos minutos incrementados
              return                             ; fim

; ---------------------------------------------------------------------------------------------
; SUBROTINA PARA SOAR O ALARME
; ---------------------------------------------------------------------------------------------
soar_alarme   btfsc    flag1,falarmstop          ; se o alarme foi interrompido,
              return                             ; sai sem soar o alarme
              bsf      PABuff,5                  ; beep
              movff    PABuff,LATA
              call     tmp_alarme                ; intervalo
              bcf      PABuff,5                  ; beep
              movff    PABuff,LATA
              call     tmp_alarme                ; intervalo
              bsf      PABuff,5                  ; beep
              movff    PABuff,LATA
              call     tmp_alarme                ; intervalo
              bcf      PABuff,5                  ; beep
              movff    PABuff,LATA
              call     tmp_alarme                ; intervalo
              bsf      PABuff,5                  ; beep
              movff    PABuff,LATA
              call     tmp_alarme                ; intervalo
              bcf      PABuff,5                  ; beep
              movff    PABuff,LATA
              call     tmp_alarme                ; intervalo
              call     tmp_alarme                ; intervalo
              call     tmp_alarme                ; intervalo
              return
tmp_alarme    movlw   .90                        ; intervalo entre beep e silencio
              call    delay_ms                   ;
              return                             ;
; ---------------------------------------------------------------------------------------------
; SUBROTINA DE PROCESSAMENTO DO ALARME
; Compara horas e minutos do alarme contra as horas e minutos correntes e, se bater,
; liga o flag de disparo do alarme. 
; O alarme sera disparado na ronda da rotina principal e so vai parar quando passar novamente
; por esta subrotina e os minutos tiverem mudado.
; ---------------------------------------------------------------------------------------------
proc_alarme   movf     dechour,w                 ; pega dezenas das horas
              subwf    al_hour,w                 ; subtrai das dezenas das horas do alarme
              btfss    STATUS,Z                  ; verifica o resultado
              goto     proc_alarme1              ; se nao deu zero, entao cai fora
;
              movf     dechour1,w                ; pega as unidades das horas
              subwf    al_hour1,w                ; subtrai das unidades das horas do alarme
              btfss    STATUS,Z                  ; verifica o resultado
              goto     proc_alarme1              ; se nao deu zero, entao cai fora
;
              movf     decmin,w                  ; pega as dezenas dos minutos 
              subwf    al_min,w                  ; subtrai das dezenas dos minutos do alarme
              btfss    STATUS,Z                  ; verifica o resultado
              goto     proc_alarme1              ; se nao deu zero, entao cai fora
;
              movf     decmin1,w                 ; pega as unidades de minutos 
              subwf    al_min1,w                 ; subtrai das unidades dos minutos do alarme
              btfss    STATUS,Z                  ; verifica o resultado
              goto     proc_alarme1              ; se nao deu zero, entao cai fora
;
              bsf      flag1,alarmon             ; senao, liga flag do alarme
              return                             ; fim
;
proc_alarme1  btfss    flag1,alarmon             ; se o alarme ja estava desligado,
              return                             ; sai fora, senao..
              call     prox_repete               ; calcula a proxima hora para disparar
              bcf      flag1,alarmon             ; desliga o alarme
              bcf      flag1,falarmstop          ; libera o som do alarme para a proxima vez
              return                             ; fim

; ---------------------------------------------------------------------------------------------
; CALCULO DA PROXIMA REPETICAO DO ALARME
; ---------------------------------------------------------------------------------------------
prox_repete    bcf     INTCON,TMR0IE             ; desabilita interrupcao de TIMER0 
               movff   rpt_min,DIGIT1            ; converte minutos da repetica a binario
               movff   rpt_min1,DIGIT2
               movlw   2
               call    dec2bin
               movff   REGA0,rptmin

               movff   rpt_hour,DIGIT1           ; converte horas da repeticao a binario
               movff   rpt_hour1,DIGIT2
               movlw   2
               call    dec2bin
               movff   REGA0,rpthour

               movff   al_min,DIGIT1             ; converte minutos do alarme a binario
               movff   al_min1,DIGIT2
               movlw   2
               call    dec2bin
               movff   REGA0,almin

               movff   al_hour,DIGIT1            ; converte horas do alarme a binario
               movff   al_hour1,DIGIT2
               movlw   2
               call    dec2bin
               movff   REGA0,alhour

               movf    rptmin,w
               addwf   almin,w
               movwf   almin
               movlw   .60
               cpfsgt  almin
               goto    vzero
               subwf   almin,f
vaium          incf    alhour,f
               goto    nvaium
vzero          movlw   .60
               cpfseq  almin
               goto    nvaium
               incf    alhour,f
               clrf    almin
;
nvaium         movf    rpthour,w
               addwf   alhour,w
               movwf   alhour
               movlw   .24
               cpfsgt  alhour
               goto    hzero
               subwf   alhour,f
               goto    virou
hzero          movlw   .24
               cpfseq  alhour
               goto    virou
               clrf    alhour
virou          call    clearA
               movff   alhour,REGA0
               call    bin2dec
               movff   DIGIT9,al_hour
               movff   DIGIT10,al_hour1
               call    clearA
               movff   almin,REGA0
               call    bin2dec
               movff   DIGIT9,al_min
               movff   DIGIT10,al_min1
;
               movlw   HIGH(intCT)               ; recarrega TIMER0                                                                                                                                                                                                                     movlw     HIGH(intCT)                    ;
               movwf   TMR0H
               movlw   LOW(intCT)
               movwf   TMR0L
               bcf     INTCON,TMR0IF
               bsf     INTCON,TMR0IE             ; habilita interrupcao de TIMER0
               return

; ---------------------------------------------------------------------------------------------
; DESLIGA FLAGS INDICADORES DE FUNCOES ATIVAS PARA A INTERRUPCAO DE TIMER0
; ---------------------------------------------------------------------------------------------
flagsoff      bcf      flag1,fhoras              ; desliga o flag de modificacao/exibicao das horas
              bcf      flag1,falarme             ; desliga o flag de modificacao/exibicao do alarme
              bcf      flag1,frepeat             ; desliga display/set repeticao,
              bcf      flag2,frelec              ; desliga modificacao  de ciclos do rele      
              bcf      flag2,freleii             ; desliga modificacao de intervalo ligado
              bcf      flag2,freleid             ; desliga modificacao de intervalo desligado
              return                             ;

; ---------------------------------------------------------------------------------------------
; RETARDO PARA BOTOES
; ---------------------------------------------------------------------------------------------
_debounce     movlw    KEY_DEBOUNCE
              call     delay_ms
              movlw    KEY_DEBOUNCE
              call     delay_ms
              return
; ---------------------------------------------------------------------------------------------
; SALVAMENTO E RESTAURACAO DOS REGISTRADORES ARITMETICOS
; Antes de chamar esta subrotina, FSR1 e FSR2 devem ser salvos.
; ---------------------------------------------------------------------------------------------
mathsave     lfsr     FSR1,REGA3                 ; origem
             lfsr     FSR2,SV_MATH               ; destino
             movlw    .26                        ; tamanho
             movwf    svct                       ; 
mathsv1      movf     POSTINC1,w                 ; pega da origem
             movwf    POSTINC2                   ; move para o destino
             decfsz   svct,f                     ; verifica o tamanho
             goto     mathsv1                    ; move o proximo
             return                              ;
mathrst      lfsr     FSR1,REGA3                 ; destino
             lfsr     FSR2,SV_MATH               ; origem
             movlw    .26                        ; tamanho
             movwf    svct                       ; 
mathrs1      movf     POSTINC2,w                 ; pega da origem
             movwf    POSTINC1                   ; move para o destino
             decfsz   svct,f                     ; verifica o tamanho
             goto     mathrs1                    ; move o proximo
             return         
; ---------------------------------------------------------------------------------------------
;        TRATAMENTO DE INTERRUPCOES EM PRIMEIRO NIVEL
; ---------------------------------------------------------------------------------------------
RTCisr   btfsc     INTCON,TMR0IF                 ; verifica quem interrompeu
         goto      intTMR0                       ; se TIMER0, vai tratar...
         btfsc     PIR1,TMR1IF                   ; verifica quem interrompeu
         goto      intTMR1                       ; se TIMER1, vai tratar...
         retfie                                  ; senao, ignora ourto tipo de interrupcao
;
; ---------------------------------------------------------------------------------------------
;        INTERRUPCAO DE TIMER0
;        Nao modifique as instrucoes deste segmento de programa ou vai se arrepender.
; ---------------------------------------------------------------------------------------------
intTMR0  swapf    Wsave1                         ; salva W
         movff    STATUS,Ssave1                  ; salva STATUS
         movff    FSR0,Sfsr0_1                   ; salva FSR0
         movff    FSR1,Sfsr1_1                   ; salva FSR1
         call     mathsave                       ; salva registradores aritmeticos
;
         bcf      INTCON,TMR0IE                  ; desabilita interrupcao de TIMER0
;
         movlw    4                              ; digito inicial
         movwf    dig_atual                      ; carrega o digito
;
;        SELECIONA O CATODO
         movlw    0x18                           ; mascara dos catodos
         xorwf    catodo,f                       ; chaveia os catodos
         movlw    b'00100001'                    ; preserva o som do alarme e o estado do rele
         andwf    PABuff,f                       ; limpa o buffer preservando o som do alarme
         movf     catodo,w                       ; pega o catodo corrente
         iorwf    PABuff,f                       ; adiciona o catodo no buffer
         movff    PABuff,LATA                   ; envia o buffer para o PORTA
;
         clrf     port1                          ; limpa o buffer do PORT do display
         clrf     port2                          ; limpa o buffer do PORT do display
;
         lfsr     FSR1,decmin1                   ; aponta para o buffer default
         btfss    flag1,fhoras                   ; verifica o flag de horas
         goto     vf1                            ; se flag1<fhoras> = 0 --> vai ver outro
         goto     nxlay                          ; prossegue
;
vf1      btfss    flag1,frepeat                  ; verifica o flag de repeticao
         goto     vf2                            ; se flag1<repeat> = 0 --> vai ver outro
         lfsr     FSR1,rpt_min1                  ; senao, aponta para o buffer de repeticao
         goto     nxlay                          ; prossegue
;
vf2      btfss    flag1,falarme                  ; verifica o flag do alarme
         goto     vf4                            ; assume o buffer default
         lfsr     FSR1,al_min1                   ; senao, aponta para o buffer do alarme
         goto     nxlay                          ; prossegue
;
vf4      btfss    flag2,freleii                   ; verifica o flag do intervalo de rele ligado
         goto     vf5                            ; assume o buffer default
         lfsr     FSR1,relelmu                   ; senao, aponta para o buffer do intervalo de rele ligado
         goto     nxlay                          ; prossegue
;
vf5      lfsr     FSR1,decmin1                   ; aponta para o buffer default
         btfss    flag2,freleid                  ; verifica o flag do intervalo de rele desligado
         goto     nxlay                          ; assume o buffer default
         lfsr     FSR1,reledmu                   ; senao, aponta para o buffer do intervalo de rele desligado
;
;        OBTEM O BITMAP DE UM CARACTER
nxlay    movlw    HIGH(chrmap)                   ; endereco dos layouts de caracteres              
         movwf    PCLATH                         ; carrega PCLATH
         movf     POSTDEC1,w                     ; pega o numero para mostrar no diaplay
         movwf    R15                            ; salva uma copia do numero em R15
         movwf    R14                            ; salva outra copia do numero em R14
         bcf      STATUS,C                       ; limpa o carry
         rlcf     R15,f                          ; multiplica por 2 porque tem que ser endereco par
         movf     R15,w                          ; pela o deslocamento
         call     chrmap                         ; pega layout do numero
         clrf     PCLATH                         ; restaura o PCLATH
         movwf    bitmap                         ; salva o bitmap
;
;        NAO DEIXA APARECER ZERO NO DIGITO 1 DO DISPLAY (FICA HORRIVEL)
         movlw    1                              ; W=1
         cpfseq   dig_atual                      ; verifica se dig_atual == 1
         goto     prmseg                         ; se dig_atual != 1, continua, senao,
         movf     R14,w                          ; pega o numero
         xorlw    0x00                           ; teste para W == 0
         btfsc    STATUS,Z                       ; se W != 0, continua, senao,
         clrf     bitmap                         ; limpa bitmap porque nao pode aparecer zero no digito 1
        
;
;        APONTA PARA O PRIMEIRO SEGMENTO DE UM NOVO DIGITO
prmseg   movff    dig_atual,segptr               
         movf     dig_atual,w
         addwf    segptr,f
         addwf    segptr,f
         addwf    segptr,f
         addwf    segptr,f
         addwf    segptr,f
         addwf    segptr,f
         movlw    .7
         movwf    segnum
         subwf    segptr,f
;
;        OBTEM UM SEGMENTO DE UM DIGITO
nxseg    movlw    HIGH(segmap)                   
         movwf    PCLATH                         ; carrega PCLATH
         movf     segptr,w                       ; pega o apontador de segmentos em segmap
         movwf    R15                            ; salva apontador
         bcf      STATUS,C                       ; limpa o carry
         rlcf     R15,f                          ; multiplica por 2
         movf     R15,w                          ; W=apontador
         call     segmap                         ; vai buscar o descritor de segmento
         clrf     PCLATH                         ; 
         movwf    segdesc                        ; salva descritor de segmento
         incf     segptr,f                       ; avanca o apotador de segmentos em segmap
;
;        VERIFICA NO BITMAP SE O SEGMENTO DEVE SER ACESO
getbitm  rrcf     bitmap,f                       ; segmento no bitmap
         btfsc    STATUS,C                       ; testa o segmento no bitmap
         goto     vkatod                         ; se deve estar aceso, vai ver o catodo,
         decfsz   segnum,f                       ; ve se e o ultimo segmento do digito corrente
         goto     nxseg                          ; pega o proximo segmento segmento
         goto     nxdig                          ; se for o ultimo, pega o proximo digito
;
;        SELECIONA O SEGMENTO, VERIFICANDO SE O SEGMENTO PERTENCE AO CATODO CORRENTE
vkatod   btfss    catodo,4                       ; teste do catodo 2
         goto     vcat1                          ; se for a vez do catodo 1, entao vai testar
         btfsc    segdesc,0                      ; teste do catodo para o segmento
         goto     pegele                         ; se for catodo 2, ok
vcat1    btfss    catodo,3                       ; teste do catodo 1
         goto     nserv                          ; esse segmento deve permacecer apagado
         btfss    segdesc,0                      ; teste do catodo para o segmento
         goto     pegele                         ; se for catodo 1, ok
nserv    decfsz   segnum,f                       ; conta o segmento
         goto     nxseg                          ; esse segmento nao vai ser aceso
         goto     nxdig                          ; se acabou os segmentos, vai para proximo digito
pegele   movff    segdesc,R14                    ; pega o descritor do segmento
         rrcf     R14,f                          ; retira o bit "z"
         movlw    0x07                           ; mascara de limpeza  
         andwf    R14,f                          ; R14 = 0000 0NNN
portset  bcf      STATUS,C                       ;  C = 0
         clrf     R1                             ; R1 = 0000 0000
         bsf      R1,0                           ; R1 = 0000 0001
         movlw    0x00                           ;  W = 0000 0000
         xorwf    R14,w                          ;  W = 0000 0???
         btfsc    STATUS,Z                       ; se deve ligar o bit zero de R1
         goto     poset1                         ; nao faz nada porque ja esta ligado, senao...
poset    rlcf     R1,f                           ; posiciona o bit da porta
         bcf      STATUS,C                       ; C = 0 para nao entrar sujeira em R1
         decfsz   R14,f                          ; proximo bit
         goto     poset                          ; 
poset1   movf     R1,w                           ; pega valor para enviar a porta
         btfsc    segdesc,5                      ; qual porta "pp" em segdesc ?
         goto     porta2                         ; se porta 2, continua, 
;
;        CONFIGURA I/O PARA ACENDER SEGMENTOS
porta1   iorwf    port1,f                        ; senao, assume porta 1
         decfsz   segnum,f                       ; verifica qual o segmento
         goto     nxseg                          ; se ainda ha segmentos neste digito
         goto     nxdig                          ; senao, vai pegar outro digito
porta2   iorwf    port2,f                        ; liga o bit na porta 2
         decfsz   segnum,f                       ; se ainda ha segmentos neste digito
         goto     nxseg                          ; entao vai pegar o proximo segmento,
         goto     nxdig                          ; senao, vai pegar outro digito
;
;        SELECIONA O PROXIMO DIGITO OU, SE FOI O ULTIMO DIGITO, ACENDE OS SEGMENTOS 
;        PARA O CATODO CORRENTE
nxdig    decfsz   dig_atual,f                    ; proximo digito
         goto     nxlay                          ; se ainda ha digitos, vai processar,
         movff    port1,LATB                    ; mostra segmentos do PORTB
         movff    port2,LATD                    ; mostra segmentos do PORTD
;
;        FIM DA INTERRUPCAO DE TIMER0
intT0f   movlw    HIGH(intCT)
         movwf    TMR0H
         movlw    LOW(intCT)
         movwf    TMR0L
;
         call     mathrst                        ; restaura registradores aritmeticos
         movff    Sfsr0_1,FSR0                   ; restaura FSR0
         movff    Sfsr1_1,FSR1                   ; restaura FSR1
         movff    Ssave1,STATUS                  ; restaura STATUS
         swapf    Wsave1,w                       ; restaura W
;
         bcf      INTCON,TMR0IF
         bsf      INTCON,TMR0IE                  ; habilita interrupcao de TIMER0

         retfie
;

; ---------------------------------------------------------------------------------------------
;        INTERRUPCAO DE TIMER1
; ---------------------------------------------------------------------------------------------
intTMR1 bcf        INTCON,TMR0IE                 ; desabilita interrupcao de TIMER0
;
        clrf      TMR1L                          ; limpa TMR1L para nao ter perigo de estourar
        movlw     HIGH(carga)                    ; pre carga
        movwf     TMR1H                          ; para 1 segundo
        movlw     LOW(carga)                     ; pre carga
        movwf     TMR1L                          ; para 1 segundo
;
        swapf      Wsave                         ; salva W
        movff      STATUS,Ssave                  ; salva STATUS
        movff      FSR0,Sfsr0_1                  ; salva FSR0
        movff      FSR1,Sfsr1_1                  ; salva FSR1
        call       mathsave                      ; salva registradores aritmeticos
;
        movlw      0x08                          ; mascara dos LEDS
        xorwf      ledsec,f                      ; vira os LEDS
        movf       ledsec,w                      ; valor acende/apaga LEDS
        movwf      LATC                         ; acende/apaga
;
        movff       decsec,DIGIT1
        movff       decsec1,DIGIT2
        movlw       .2
        call        dec2bin
        movff       REGA0,ss

        movff       decmin,DIGIT1
        movff       decmin1,DIGIT2
        movlw       .2
        call        dec2bin
        movff       REGA0,mm

        movff       dechour,DIGIT1
        movff       dechour1,DIGIT2
        movlw       .2
        call        dec2bin
        movff       REGA0,hh

        incf        ss,f
        movlw       .59
        cpfsgt      ss
        goto        RTCrep
        clrf        ss

        incf        mm,f
        movlw       .59
        cpfsgt      mm
        goto        RTCrep
        clrf        mm

        incf        hh,f
        movlw       .23
        cpfsgt      hh
        goto        RTCrep        
        clrf        hh

RTCrep  call        clearA
        movff       ss,REGA0
        call        bin2dec
        movff       DIGIT9,decsec
        movff       DIGIT10,decsec1
 
        call        clearA
        movff       mm,REGA0
        call        bin2dec
        movff       DIGIT9,decmin
        movff       DIGIT10,decmin1
 
        call        clearA
        movff       hh,REGA0
        call        bin2dec
        movff       DIGIT9,dechour
        movff       DIGIT10,dechour1
;
; -----------------------------------------------------------------------------
;       incrementa o contador de rele ligado/desligado se assim for necessario
        btfsc       flag1,frelesta
        goto        t1relel
        goto        t1reled
;
t1relel incf        relelcs,f
        movlw       .59
        cpfsgt      relelcs
        goto        RTCfim
        clrf        relelcs
        incf        relelcm,f
        movlw       .59
        cpfsgt      relelcm
        goto        RTCfim
        clrf        relelcm
        incf        relelch,f
        movlw       .23
        cpfsgt      relelch
        goto        RTCfim
        clrf        relelch
        goto        RTCfim
;
t1reled incf        reledcs,f
        movlw       .59
        cpfsgt      reledcs
        goto        RTCfim
        clrf        reledcs
        incf        reledcm,f
        movlw       .59
        cpfsgt      reledcm
        goto        RTCfim
        clrf        reledcm
        incf        reledch,f
        movlw       .23
        cpfsgt      reledch
        goto        RTCfim
        clrf        reledch
;
; ----------------------------------------------------------------------
;
RTCfim  movlw     HIGH(intCT)                    ; recarrega TIMER0 
        movwf     TMR0H
        movlw     LOW(intCT)
        movwf     TMR0L

        call      mathrst                        ; restaura registradores aritmeticos
        movff     Sfsr0_1,FSR0                   ; restaura FSR0
        movff     Sfsr1_1,FSR1                   ; restaura FSR1
        movff     Ssave,STATUS                   ; restaura STATUS
        swapf     Wsave,w                        ; restaura W

        bcf       INTCON,TMR0IF                  ; limpa flag de interrupcao de TIMER0
        bcf       PIR1, TMR1IF                   ; limpa flag de interrupcao do TIMER1
        bcf       INTCON,TMR0IF                  ; descarta interrupcao de TIMER0
        bsf       INTCON,TMR0IE                  ; habilita interrupcao de TIMER0

        retfie                     




; ---------------------------------------------------------------------------------------------
; Converte numero binario de 32 bits para decimal 
; Entrada: REGA contem o numero binario de 32 bits
; Saida:   DIGIT1 a DIGIT10 contem o numero decimal (0x0n0n0n0n0n0n0n0n0n0n)
; ---------------------------------------------------------------------------------------------
bin2dec       clrf     MTEMP                     ; Reset sign flag
              call     absa                      ; Make REGA positive
              btfsc    STATUS,C
              return                             ; Overflow
              call     clrdig                    ; Clear all digits
              movlw    D'32'                     ; Loop counter
              movwf    MCOUNT
b2dloop       rlcf     REGA0,f                   ; Shift msb into carry
              rlcf     REGA1,f
              rlcf     REGA2,f
              rlcf     REGA3,f                   ;
              lfsr     FSR0,DIGIT10              ; Pointer to digits
              movlw    D'10'                     ; 10 digits to do
              movwf    DCOUNT
adjlp         rlcf     INDF0,f                   ; Shift digit and carry 1 bit left
              movlw    D'10'
              subwf    INDF0,w                   ; Check and adjust for decimal overflow
              btfsc    STATUS,C
              movwf    INDF0
              movf     POSTDEC0,w                ; falso move para decrementar FSR0
              decfsz   DCOUNT,f
              goto     adjlp
              decfsz   MCOUNT,f                  ; Next bit
              goto     b2dloop
              btfsc    MTEMP,0                   ; Check sign
              bsf      DSIGN,0                   ; Negative
              bcf      STATUS,C
              return
absa          rlcf     REGA3,w
              btfss    STATUS,C
              return                             ; Positive
clrdig        clrf     DSIGN
              clrf     DIGIT1
              clrf     DIGIT2
              clrf     DIGIT3
              clrf     DIGIT4
              clrf     DIGIT5
              clrf     DIGIT6
              clrf     DIGIT7
              clrf     DIGIT8
              clrf     DIGIT9
              clrf     DIGIT10
              return

clearA        clrf     REGA3
              clrf     REGA2
              clrf     REGA1
              clrf     REGA0
              return
; ---------------------------------------------------------------------------------------------
; Converte numero decimal a binario de 32 bits
; Entrada: DIGIT1 a DIGIT10 contem o numero decimal (0x0n0n0n0n0n0n0n0n0n0n)
;          W contem o numero de digitos a converter
; Saida:   REGA contera o numero binario de 32 bits
; ---------------------------------------------------------------------------------------------
dec2bin    movwf    MTEMP                        ; Save digit count
           movlw    D'32'                        ; Outer bit loop counter
           movwf    MCOUNT
d2blp1     lfsr     FSR0,DSIGN                   ; Set up pointer to MSD, isto e, DIGIT1-1
           movf     MTEMP,w                      ; Inner digit loop counter
           movwf    DCOUNT
           movlw    D'10'
           bcf      STATUS,C                     ; Bring in '0' bit into MSD
           movff    STATUS,SVStatus
d2blp2     incf     FSR0L,f
           movff    SVStatus,STATUS
           btfsc    STATUS,C
           addwf	INDF0,f                      ; Add 10 if '1' bit from prev digit
           rrcf     INDF0,f                      ; Shift out LSB of digit
           movff    STATUS,SVStatus
           decfsz   DCOUNT,f                     ; Next L.S. Digit
           goto     d2blp2
           rrcf     REGA3,f                      ; Shift in carry from digits
           rrcf     REGA2,f
           rrcf     REGA1,f
           rrcf     REGA0,f
           decfsz   MCOUNT,f                     ; Next bit
           goto     d2blp1
           movf     INDF0,w                      ; Check for overflow
           addlw    0xFF
           btfss    STATUS,C
           rlcf     REGA3,w
           btfsc    STATUS,C
           return
           btfsc    DSIGN,0                      ; Check result sign
           call     negatea                      ; Negative
           return
negatea    movf     REGA3,w                      ; Save sign in w
           andlw    0x80
           comf     REGA0,f                      ; 2's complement
           comf     REGA1,f
           comf     REGA2,f
           comf     REGA3,f
           incfsz	REGA0,f
           goto     nega1
           incfsz   REGA1,f
           goto     nega1
           incfsz   REGA2,f
           goto     nega1
           incf     REGA3,f
nega1      incf     MTEMP,f                      ; flip sign flag
           addwf    REGA3,w                      ; Return carry set if -2147483648
           return
;
;
; ---------------------------------------------------------------------------------------------------
;
;
         #include "delay_us.inc"                 ; arquivo de origem --> "delay_us.inc"
         #include "delay_ms.inc"                 ; arquivo de origem --> "delay_ms.inc"
;
;
; ---------------------------------------------------------------------------------------------------
; BITMAP DOS CARACTERS PARA O DISPLAY DE 7 SEGMENTOS
        ORG   0x1600
chrmap  ADDWF   PCL,f
;               .gfedcba
        DT    b'00111111'           ; 0x00 --> char "0"
	    DT    b'00000110'           ; 0x01 --> char "1"
	    DT    b'01011011'           ; 0x02 --> char "2"
	    DT    b'01001111'           ; 0x03 --> char "3"
	    DT    b'01100110'           ; 0x04 --> char "4"
	    DT    b'01101101'           ; 0x05 --> char "5"
	    DT    b'01111101'           ; 0x06 --> char "6"
	    DT    b'00000111'           ; 0x07 --> char "7"
	    DT    b'01111111'           ; 0x08 --> char "8"
	    DT    b'01101111'           ; 0x09 --> char "9"
	    DT    b'01110111'           ; 0x0A --> char "A"
	    DT    b'01111100'           ; 0x0B --> char "b"
	    DT    b'00111001'           ; 0x0C --> char "C"
	    DT    b'01011110'           ; 0x0D --> char "d"
	    DT    b'01111001'           ; 0x0E --> char "E"
	    DT    b'01110001'           ; 0x0F --> char "F"
        DT    b'01011100'           ; 0x10 --> char "bolinha"
        DT    b'01110011'           ; 0x11 --> char "S"
        DT    b'01101101'           ; 0x12 --> char "P"
	    DT    b'00000000'           ; empty
        DT    0x00
;
; ---------------------------------------------------------------------------------------------------
; MAPEAMENTO DOS SEGMENTOS
; Cada segmento sera conectado a um bit de uma porta e tambem possuira um catodo.
; A tabela abaixo associa cada segmento a um bit de porta de I/O e um catodo.
;
;  BIT  
;  7:6   - nao usados
;  5:4   - PORT : 00=PORTB,  01=PORTC,  10=PORTD
;  3:1   - bit do PORT (0 a 7)
;    0   - numero do Catodo:  K1=0  K2=1
; ---------------------------------------------------------------------------------------------------
            org     0x1640
segmap      ADDWF   PCL,f
            DT    b'00001100'                  ; segmento 0    a   Digito 1
	        DT    b'00001111'                  ; segmento 1    b
            DT    b'00001001'                  ; segmento 2    c
            DT    b'00001010'                  ; segmento 3    d
            DT    b'00001011'                  ; segmento 4    e
            DT    b'00000000'                  ; segmento 5    f
            DT    b'00001101'                  ; segmento 6    g
;
            DT    b'00000011'                  ; segmento 7    a   Digito 2
            DT    b'00000111'                  ; segmento 8    b 
            DT    b'00000101'                  ; segmento 9    c
            DT    b'00000100'                  ; segmento 10   d
            DT    b'00001000'                  ; segmento 11   e
            DT    b'00000010'                  ; segmento 12   f
            DT    b'00000110'                  ; segmento 13   g
;
            DT    b'00000000'                  ; segmento 14   a   Digito 3
            DT    b'00101110'                  ; segmento 15   b
            DT    b'00101100'                  ; segmento 16   c
            DT    b'00101101'                  ; segmento 17   d
            DT    b'00101011'                  ; segmento 18   e
            DT    b'00000001'                  ; segmento 19   f
            DT    b'00101111'                  ; segmento 20   g
;
            DT    b'00100101'                  ; segmento 21   a   Digito 4
            DT    b'00101001'                  ; segmento 22   b
            DT    b'00100111'                  ; segmento 23   c
            DT    b'00100110'                  ; segmento 24   d
            DT    b'00101010'                  ; segmento 25   e
            DT    b'00100100'                  ; segmento 26   f
            DT    b'00101000'                  ; segmento 27   g
;
;
;
         END



